
BEGIN;

ALTER TABLE dbmail_messages ADD COLUMN seq BIGINT NOT NULL default '0';

CREATE INDEX dbmail_messages_seq_index ON dbmail_messages(seq);

INSERT INTO dbmail_upgrade_steps (from_version, to_version) values (32001, 32003);

COMMIT;
